// © 2020 ETH Zurich, Mechanics and Materials Lab
// © 2020 California Institute of Technology
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

#include "ae108/cpppetsc/SequentialComputePolicy_fwd.h"
// #include <mpi.h>
#include <boost/mpi.hpp>
#include <boost/mpi/collectives.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/shared_ptr.hpp>
#include <petscsys.h>

namespace ae108 {
namespace cpppetsc {

/**
 * @brief The policies configure the wrapper classes (error handling,
 * communicator).
 */
struct SequentialComputePolicy {
  static inline MPI_Comm communicator();

  static inline void handleError(const PetscErrorCode code);

  static inline bool isPrimaryRank();
};

/********************************************************************
 *  implementations
 *******************************************************************/

MPI_Comm SequentialComputePolicy::communicator() { return PETSC_COMM_SELF; }

void SequentialComputePolicy::handleError(const PetscErrorCode code) {
  CHKERRABORT(communicator(), code);
}

bool SequentialComputePolicy::isPrimaryRank() { return true; }
} // namespace cpppetsc
} // namespace ae108